------------------------------------------------------------------------
--
-- Functions to create, drop and alter database users.
--
-- This functions prevent granting CREATEUSER to just a little operator
-- account because this gives SUPERUSER privileges!!!
--
-- They are simulating the CREATE USER, ALTER USER and DROP USER system
-- privileges.
--
-- This script has to be executed under a superuser (e.g. postgres).
--
-- The procedures are visible under the public schema but NOT executable.
-- They are added to the amdin_user group. A operator user who should be
-- able to execute this functions has to be added to this group.
--
-- e.g ALTER GROUP admin_user ADD USER operator;
--
-- This script does NOT work before PostgreSQL 7.4.!!!
--
-- Copyright (c) 2004 under GPL by Oli Sennhauser
-- Any feedback is welcome to: oli_dot_sennhauser_at_bluewin_dot_ch
--
------------------------------------------------------------------------


------------------------------------------------------------------------
CREATE FUNCTION public.create_user(
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.usesysid%TYPE,
  text,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_group.groname%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  boolean)
-- $1 - Username
-- $2 - Sysid
-- $3 - ENCRYPTED or UNENCRYPTED
-- $4 - Password
-- $5 - Groupname
-- $6 - Expiry date
-- $7 - Creation of the schema
--
-- Example: SELECT create_user('testuser', NULL, NULL, '4XKdD3W',
--          'admin_user', NULL, TRUE);
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  sql VARCHAR;
  msg VARCHAR;
BEGIN

  sql := ''CREATE USER '' || quote_ident($1) || '' '';
  msg := ''CREATE USER'';

  IF $2 IS NOT NULL THEN
    sql := sql || ''SYSID '' || $2 || '' '';
  END IF;

  IF $4 IS NOT NULL THEN
    IF UPPER($3) = ''UNENCRYPTED'' THEN
      sql := sql || ''UNENCRYPTED '';
    ELSE
      sql := sql || ''ENCRYPTED '';
    END IF;

    sql := sql || ''PASSWORD '' || quote_literal($4) || '' '';
  END IF;

  IF $5 IS NOT NULL THEN
    sql := sql || ''IN GROUP '' || $5 || '' '';
  END IF;

  IF $6 IS NOT NULL THEN
    sql := sql || ''VALID UNTIL '' || quote_literal($6::timestamp::text);
  END IF;

  IF $7 IS NOT FALSE THEN
    sql := sql || ''; CREATE SCHEMA AUTHORIZATION '' || quote_ident($1)
        || '';'';
    msg := msg || '', CREATE SCHEMA'';
  END IF;

  EXECUTE sql;
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.create_user(
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.usesysid%TYPE,
  text,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_group.groname%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  boolean)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.create_user(
  pg_catalog.pg_shadow.usename%TYPE,
  pg_catalog.pg_shadow.passwd%TYPE,
  boolean)
-- $1 - Username
-- $2 - Password
-- $3 - Creation of the schema
--
-- Example: SELECT create_user('testuser', '4XKdD3W', FALSE);
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  msg VARCHAR;
BEGIN
  msg := create_user($1, NULL, ''ENCRYPTED'', $2, NULL, NULL, $3);
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.create_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE,
  boolean)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.create_user(
  pg_catalog.pg_shadow.usename%TYPE,
  pg_catalog.pg_shadow.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE)
-- $1 - Username
-- $2 - Password
-- $3 - Expiry date
--
-- Example: SELECT create_user('testuser', '4XKdD3W',
--          '2004-12-31'::abstime);
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  msg VARCHAR;
BEGIN
  msg := create_user($1, NULL, ''ENCRYPTED'', $2, NULL, $3, FALSE);
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.create_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.create_user(
  pg_catalog.pg_shadow.usename%TYPE,
  pg_catalog.pg_shadow.passwd%TYPE)
-- $1 - Username
-- $2 - Password
--
-- Example: SELECT create_user('testuser', '4XKdD3W');
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  msg VARCHAR;
BEGIN
  msg := create_user($1, NULL, ''ENCRYPTED'', $2, NULL, NULL, FALSE);
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.create_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.drop_user(
  pg_catalog.pg_shadow.usename%TYPE,
  boolean)
-- $1 - Username
-- $2 - Drop schema
--
-- Example: SELECT drop_user('testuser', FALSE);
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  msg VARCHAR;
  sql VARCHAR;
  cnt NUMERIC;
BEGIN

  sql := ''DROP USER '' || quote_ident($1) || '';'';
  msg := ''DROP USER'';

  IF $2 THEN
    SELECT COUNT(nspname) INTO cnt
    FROM pg_namespace
    WHERE nspname = quote_ident($1);

    IF cnt = 1 THEN
      sql := sql || ''DROP SCHEMA '' || quote_ident($1) || '' CASCADE;'';
      msg := msg || '', DROP SCHEMA'';
    END IF;
  END IF;

  EXECUTE sql;
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.drop_user (
  pg_catalog.pg_user.usename%TYPE,
  boolean)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.rename_user(
  pg_catalog.pg_shadow.usename%TYPE,
  pg_catalog.pg_shadow.usename%TYPE)
-- $1 - old username
-- $2 - new username
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  msg VARCHAR;
  sql VARCHAR;
BEGIN

  sql := ''ALTER USER '' || quote_ident($1) || '' RENAME TO '' ||
         quote_ident($2);

  EXECUTE sql;
  msg := ''ALTER USER'';

  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.rename_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.usename%TYPE)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE FUNCTION public.alter_user(
  pg_catalog.pg_user.usename%TYPE,
  text,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  pg_catalog.pg_settings.name%TYPE,
  pg_catalog.pg_settings.setting%TYPE)
-- $1 - username
-- $2 - text ENCRYPTED or UNECRYPTED
-- $3 - Password
-- $4 - Valid until date
-- $5 - Parameter name
-- $6 - Parameter value
------------------------------------------------------------------------
RETURNS varchar AS '
DECLARE
  sql VARCHAR;
  msg VARCHAR;
  val VARCHAR;
BEGIN

  sql := ''ALTER USER '' || quote_ident($1) || '' '';
  msg := ''ALTER USER'';

  IF $3 IS NOT NULL THEN
    IF UPPER($2) = ''UNENCRYPTED'' THEN
      sql := sql || '' UNENCRYPTED '';
    ELSE
      sql := sql || '' ENCRYPTED '';
    END IF;

    sql := sql || ''PASSWORD '' || quote_literal($3) || '' '';
  END IF;

  IF $4 IS NOT NULL THEN
    sql := sql || ''VALID UNTIL '' || quote_literal($4::timestamp::text);
  END IF;

  IF $5 IS NOT FALSE THEN
    IF $6 IS NULL
      val := ''DEFAULT'';
    ELSE
      val := $6;
    END IF;

    sql := sql || ''; ALTER USER '' || quote_ident($1) || '' SET '' ||
           $5 || '' TO '' || val || '';'';
    msg := msg || '', ALTER USER'';
  END IF;

  EXECUTE sql;
  RETURN msg;
END;
' LANGUAGE plpgsql
SECURITY DEFINER;

REVOKE EXECUTE ON FUNCTION public.alter_user(
  pg_catalog.pg_user.usename%TYPE,
  text,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  pg_catalog.pg_settings.name%TYPE,
  pg_catalog.pg_settings.setting%TYPE)
FROM PUBLIC;


------------------------------------------------------------------------
CREATE GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.create_user(
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.usesysid%TYPE,
  text, pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_group.groname%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  boolean)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.create_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE,
  boolean)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.create_user(
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.create_user(
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.passwd%TYPE)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.drop_user(
  pg_catalog.pg_user.usename%TYPE,
  boolean)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.rename_user (
  pg_catalog.pg_user.usename%TYPE,
  pg_catalog.pg_user.usename%TYPE)
TO GROUP admin_user;

GRANT EXECUTE ON FUNCTION public.alter_user(
  pg_catalog.pg_user.usename%TYPE,
  text,
  pg_catalog.pg_user.passwd%TYPE,
  pg_catalog.pg_shadow.valuntil%TYPE,
  pg_catalog.pg_settings.name%TYPE,
  pg_catalog.pg_settings.setting%TYPE)
TO GROUP admin_user;
